﻿Imports System.Threading.Tasks
Imports Microsoft.AspNet.Identity
Imports Microsoft.AspNet.Identity.EntityFramework
Imports Microsoft.AspNet.Identity.Owin
Imports Microsoft.Owin
Imports FrontClass.Models

Namespace FrontClass
	' Configure the application user manager used in this application. UserManager is defined in ASP.NET Identity and is used by the application.

	Public Class ApplicationUserManager
		Inherits UserManager(Of ApplicationUser)
		Public Sub New(ByVal store As IUserStore(Of ApplicationUser))
			MyBase.New(store)
		End Sub

		Public Shared Function Create(ByVal options As IdentityFactoryOptions(Of ApplicationUserManager), ByVal context As IOwinContext) As ApplicationUserManager
			Dim manager = New ApplicationUserManager(New UserStore(Of ApplicationUser)(context.Get(Of ApplicationDbContext)()))
			' Configure validation logic for usernames
			manager.UserValidator = New UserValidator(Of ApplicationUser)(manager) With {.AllowOnlyAlphanumericUserNames = False, .RequireUniqueEmail = True}
			' Configure validation logic for passwords
			manager.PasswordValidator = New PasswordValidator With {.RequiredLength = 6, .RequireNonLetterOrDigit = True, .RequireDigit = True, .RequireLowercase = True, .RequireUppercase = True}
			' Register two factor authentication providers. This application uses Phone and Emails as a step of receiving a code for verifying the user
			' You can write your own provider and plug in here.
			manager.RegisterTwoFactorProvider("PhoneCode", New PhoneNumberTokenProvider(Of ApplicationUser) With {.MessageFormat = "Your security code is: {0}"})
			manager.RegisterTwoFactorProvider("EmailCode", New EmailTokenProvider(Of ApplicationUser) With {.Subject = "Security Code", .BodyFormat = "Your security code is: {0}"})
			manager.EmailService = New EmailService()
			manager.SmsService = New SmsService()
			Dim dataProtectionProvider = options.DataProtectionProvider
			If dataProtectionProvider IsNot Nothing Then
				manager.UserTokenProvider = New DataProtectorTokenProvider(Of ApplicationUser)(dataProtectionProvider.Create("ASP.NET Identity"))
			End If
			Return manager
		End Function
	End Class

	Public Class EmailService
		Implements IIdentityMessageService
		Public Function SendAsync(ByVal message As IdentityMessage) As Task Implements IIdentityMessageService.SendAsync
			' Plug in your email service here to send an email.
			Return Task.FromResult(0)
		End Function
	End Class

	Public Class SmsService
		Implements IIdentityMessageService
		Public Function SendAsync(ByVal message As IdentityMessage) As Task Implements IIdentityMessageService.SendAsync
			' Plug in your sms service here to send a text message.
			Return Task.FromResult(0)
		End Function
	End Class
End Namespace
