﻿Imports System.Data.Entity
Imports System.Net
Imports System.Threading.Tasks
Imports System.Web.Mvc
Imports FrontClass.DAL
Imports FrontClass.Models

Namespace FrontClass.Areas.Administration.Controllers
	<Authorize(Roles := MvcApplication.AdministratorRoleName)>
	Public Class CoursesController
		Inherits Controller
		Private db As New FrontClassContext()

		' GET: Administration/Courses
		Public async Function Index() As Task(Of ActionResult)
			Return View(await db.Courses.ToListAsync())
		End Function

		' GET: Administration/Courses/Details/5
		Public async Function Details(ByVal id? As Integer) As Task(Of ActionResult)
			If id Is Nothing Then
				Return New HttpStatusCodeResult(HttpStatusCode.BadRequest)
			End If
			Dim course As Course = await db.Courses.FindAsync(id)
			If course Is Nothing Then
				Return HttpNotFound()
			End If
			Return View(course)
		End Function

		' GET: Administration/Courses/Create
		Public Function Create() As ActionResult
			Return View()
		End Function

		' POST: Administration/Courses/Create
		' To protect from overposting attacks, please enable the specific properties you want to bind to, for 
		' more details see http://go.microsoft.com/fwlink/?LinkId=317598.
		<HttpPost, ValidateAntiForgeryToken>
		Public async Function Create(<Bind(Include := "CourseId,Title,Description,EntryCode")> ByVal course As Course) As Task(Of ActionResult)
			If ModelState.IsValid Then
				db.Courses.Add(course)
				await db.SaveChangesAsync()
				Return RedirectToAction("Index")
			End If

			Return View(course)
		End Function

		' GET: Administration/Courses/Edit/5
		Public async Function Edit(ByVal id? As Integer) As Task(Of ActionResult)
			If id Is Nothing Then
				Return New HttpStatusCodeResult(HttpStatusCode.BadRequest)
			End If
			Dim course As Course = await db.Courses.FindAsync(id)
			If course Is Nothing Then
				Return HttpNotFound()
			End If
			Return View(course)
		End Function

		' POST: Administration/Courses/Edit/5
		' To protect from overposting attacks, please enable the specific properties you want to bind to, for 
		' more details see http://go.microsoft.com/fwlink/?LinkId=317598.
		<HttpPost, ValidateAntiForgeryToken>
		Public async Function Edit(<Bind(Include := "CourseId,Title,Description,EntryCode")> ByVal course As Course) As Task(Of ActionResult)
			If ModelState.IsValid Then
				db.Entry(course).State = EntityState.Modified
				await db.SaveChangesAsync()
				Return RedirectToAction("Index")
			End If
			Return View(course)
		End Function

		' GET: Administration/Courses/Delete/5
		Public async Function Delete(ByVal id? As Integer) As Task(Of ActionResult)
			If id Is Nothing Then
				Return New HttpStatusCodeResult(HttpStatusCode.BadRequest)
			End If
			Dim course As Course = await db.Courses.FindAsync(id)
			If course Is Nothing Then
				Return HttpNotFound()
			End If
			Return View(course)
		End Function

		' POST: Administration/Courses/Delete/5
		<HttpPost, ActionName("Delete"), ValidateAntiForgeryToken>
		Public async Function DeleteConfirmed(ByVal id As Integer) As Task(Of ActionResult)
			Dim course As Course = await db.Courses.FindAsync(id)
			db.Courses.Remove(course)
			await db.SaveChangesAsync()
			Return RedirectToAction("Index")
		End Function

		Protected Overrides Overloads Sub Dispose(ByVal disposing As Boolean)
			If disposing Then
				db.Dispose()
			End If
			MyBase.Dispose(disposing)
		End Sub
	End Class
End Namespace