﻿Imports System.Data.Entity
Imports System.Net
Imports System.Threading.Tasks
Imports System.Web.Mvc
Imports FrontClass.DAL
Imports FrontClass.Models

Namespace FrontClass.Areas.Administration.Controllers
	<Authorize(Roles := MvcApplication.AdministratorRoleName)>
	Public Class ModulesController
		Inherits Controller
		Private db As New FrontClassContext()

		' GET: Administration/Modules
		Public async Function Index() As Task(Of ActionResult)
			Dim modules = db.Modules.Include(Function(m) m.Course)
			Return View(await modules.ToListAsync())
		End Function

		' GET: Administration/Modules/Details/5
		Public async Function Details(ByVal id? As Integer) As Task(Of ActionResult)
			If id Is Nothing Then
				Return New HttpStatusCodeResult(HttpStatusCode.BadRequest)
			End If
            Dim [module] As FrontClass.Models.Module = Await db.Modules.FindAsync(id)
			If [module] Is Nothing Then
				Return HttpNotFound()
			End If
			Return View([module])
		End Function

		' GET: Administration/Modules/Create
		Public Function Create() As ActionResult
			ViewBag.CourseId = New SelectList(db.Courses, "CourseId", "Title")
			Return View()
		End Function

		' POST: Administration/Modules/Create
		' To protect from overposting attacks, please enable the specific properties you want to bind to, for 
		' more details see http://go.microsoft.com/fwlink/?LinkId=317598.
		<HttpPost, ValidateAntiForgeryToken>
		Public async Function Create(<Bind(Include := "ModuleId,Title,Description,CourseId")> ByVal [module] As [Module]) As Task(Of ActionResult)
			If ModelState.IsValid Then
				db.Modules.Add([module])
				await db.SaveChangesAsync()
				Return RedirectToAction("Index")
			End If

			ViewBag.CourseId = New SelectList(db.Courses, "CourseId", "Title", [module].CourseId)
			Return View([module])
		End Function

		' GET: Administration/Modules/Edit/5
		Public async Function Edit(ByVal id? As Integer) As Task(Of ActionResult)
			If id Is Nothing Then
				Return New HttpStatusCodeResult(HttpStatusCode.BadRequest)
			End If
            Dim [module] As FrontClass.Models.Module = Await db.Modules.FindAsync(id)
			If [module] Is Nothing Then
				Return HttpNotFound()
			End If
			ViewBag.CourseId = New SelectList(db.Courses, "CourseId", "Title", [module].CourseId)
			Return View([module])
		End Function

		' POST: Administration/Modules/Edit/5
		' To protect from overposting attacks, please enable the specific properties you want to bind to, for 
		' more details see http://go.microsoft.com/fwlink/?LinkId=317598.
		<HttpPost, ValidateAntiForgeryToken>
		Public async Function Edit(<Bind(Include := "ModuleId,Title,Description,CourseId")> ByVal [module] As [Module]) As Task(Of ActionResult)
			If ModelState.IsValid Then
				db.Entry([module]).State = EntityState.Modified
				await db.SaveChangesAsync()
				Return RedirectToAction("Index")
			End If
			ViewBag.CourseId = New SelectList(db.Courses, "CourseId", "Title", [module].CourseId)
			Return View([module])
		End Function

		' GET: Administration/Modules/Delete/5
		Public async Function Delete(ByVal id? As Integer) As Task(Of ActionResult)
			If id Is Nothing Then
				Return New HttpStatusCodeResult(HttpStatusCode.BadRequest)
			End If
            Dim [module] As FrontClass.Models.Module = Await db.Modules.FindAsync(id)
			If [module] Is Nothing Then
				Return HttpNotFound()
			End If
			Return View([module])
		End Function

		' POST: Administration/Modules/Delete/5
		<HttpPost, ActionName("Delete"), ValidateAntiForgeryToken>
		Public async Function DeleteConfirmed(ByVal id As Integer) As Task(Of ActionResult)
            Dim [module] As FrontClass.Models.Module = Await db.Modules.FindAsync(id)
			db.Modules.Remove([module])
			await db.SaveChangesAsync()
			Return RedirectToAction("Index")
		End Function

		Protected Overrides Overloads Sub Dispose(ByVal disposing As Boolean)
			If disposing Then
				db.Dispose()
			End If
			MyBase.Dispose(disposing)
		End Sub
	End Class
End Namespace