﻿Imports System.Data.Entity
Imports System.Net
Imports System.Threading.Tasks
Imports System.Web.Mvc
Imports FrontClass.DAL
Imports FrontClass.Models

Namespace FrontClass.Areas.Administration.Controllers
	<Authorize(Roles := MvcApplication.AdministratorRoleName)>
	Public Class StepsController
		Inherits Controller
		Private db As New FrontClassContext()

		' GET: Administration/Steps
		Public async Function Index() As Task(Of ActionResult)
			Dim steps = db.Steps.Include(Function(s) s.Module)
			Return View(await steps.ToListAsync())
		End Function

		' GET: Administration/Steps/Details/5
		Public async Function Details(ByVal id? As Integer) As Task(Of ActionResult)
			If id Is Nothing Then
				Return New HttpStatusCodeResult(HttpStatusCode.BadRequest)
			End If
			Dim [step] As [Step] = await db.Steps.FindAsync(id)
			If [step] Is Nothing Then
				Return HttpNotFound()
			End If
			Return View([step])
		End Function

		' GET: Administration/Steps/Create
		Public Function Create() As ActionResult
			ViewBag.ModuleId = New SelectList(db.Modules, "ModuleId", "Title")
			Return View()
		End Function

		' POST: Administration/Steps/Create
		' To protect from overposting attacks, please enable the specific properties you want to bind to, for 
		' more details see http://go.microsoft.com/fwlink/?LinkId=317598.
		<HttpPost, ValidateAntiForgeryToken>
		Public async Function Create(<Bind(Include := "StepId,Title,Content,ModuleId")> ByVal [step] As [Step]) As Task(Of ActionResult)
			If ModelState.IsValid Then
				db.Steps.Add([step])
				await db.SaveChangesAsync()
				Return RedirectToAction("Index")
			End If

			ViewBag.ModuleId = New SelectList(db.Modules, "ModuleId", "Title", [step].ModuleId)
			Return View([step])
		End Function

		' GET: Administration/Steps/Edit/5
		Public async Function Edit(ByVal id? As Integer) As Task(Of ActionResult)
			If id Is Nothing Then
				Return New HttpStatusCodeResult(HttpStatusCode.BadRequest)
			End If
			Dim [step] As [Step] = await db.Steps.FindAsync(id)
			If [step] Is Nothing Then
				Return HttpNotFound()
			End If
			ViewBag.ModuleId = New SelectList(db.Modules, "ModuleId", "Title", [step].ModuleId)
			Return View([step])
		End Function

		' POST: Administration/Steps/Edit/5
		' To protect from overposting attacks, please enable the specific properties you want to bind to, for 
		' more details see http://go.microsoft.com/fwlink/?LinkId=317598.
		<HttpPost, ValidateAntiForgeryToken>
		Public async Function Edit(<Bind(Include := "StepId,Title,Content,ModuleId")> ByVal [step] As [Step]) As Task(Of ActionResult)
			If ModelState.IsValid Then
				db.Entry([step]).State = EntityState.Modified
				await db.SaveChangesAsync()
				Return RedirectToAction("Index")
			End If
			ViewBag.ModuleId = New SelectList(db.Modules, "ModuleId", "Title", [step].ModuleId)
			Return View([step])
		End Function

		' GET: Administration/Steps/Delete/5
		Public async Function Delete(ByVal id? As Integer) As Task(Of ActionResult)
			If id Is Nothing Then
				Return New HttpStatusCodeResult(HttpStatusCode.BadRequest)
			End If
			Dim [step] As [Step] = await db.Steps.FindAsync(id)
			If [step] Is Nothing Then
				Return HttpNotFound()
			End If
			Return View([step])
		End Function

		' POST: Administration/Steps/Delete/5
		<HttpPost, ActionName("Delete"), ValidateAntiForgeryToken>
		Public async Function DeleteConfirmed(ByVal id As Integer) As Task(Of ActionResult)
			Dim [step] As [Step] = await db.Steps.FindAsync(id)
			db.Steps.Remove([step])
			await db.SaveChangesAsync()
			Return RedirectToAction("Index")
		End Function

		Protected Overrides Overloads Sub Dispose(ByVal disposing As Boolean)
			If disposing Then
				db.Dispose()
			End If
			MyBase.Dispose(disposing)
		End Sub
	End Class
End Namespace