﻿Imports System.Globalization
Imports System.Security.Claims
Imports System.Web.Mvc
Imports System.Web.Routing
Imports FrontClass.Models
Imports Microsoft.AspNet.Identity
Imports Microsoft.AspNet.Identity.EntityFramework

Namespace FrontClass.Filters
	''' <summary>
	''' Allows administrators and users who have a claim for the
	''' current course to access an attributed controller/action
	''' </summary>
	Public Class EnrolmentVerificationFilter
		Inherits AuthorizeAttribute
		Protected Overrides Function AuthorizeCore(ByVal httpContext As HttpContextBase) As Boolean
			' no active course
			If MvcApplication.Classroom.CurrentModule Is Nothing Then
				Return False
			End If

			' call base class to ensure user is signed in
			If Not MyBase.AuthorizeCore(httpContext) Then
				Return False
			End If

			' the user must be an administrator...
			If httpContext.User.IsInRole(MvcApplication.AdministratorRoleName) Then
				Return True
			End If

			' ...or have registered for the course
			Return UserHasClaimForCourse(httpContext)

		End Function

		Private Shared Function UserHasClaimForCourse(ByVal httpContext As HttpContextBase) As Boolean
			Dim courseId = MvcApplication.Classroom.CurrentModule.CourseId

			' check for the claim against the current course
			Dim context = New ApplicationDbContext()
			Dim userStore = New UserStore(Of ApplicationUser)(context)
			Dim userManager = New UserManager(Of ApplicationUser)(userStore)

			Dim userId = httpContext.User.Identity.GetUserId()

			Dim hasClaimForActiveCourse = userManager.GetClaims(userId).Where(Function(c) c.Type = MvcApplication.CourseRegistrationClaimUrn).Any(Function(c) c.Value = courseId.ToString(CultureInfo.InvariantCulture))

			Return hasClaimForActiveCourse
		End Function

		Protected Overrides Sub HandleUnauthorizedRequest(ByVal filterContext As AuthorizationContext)
			If Not filterContext.HttpContext.User.Identity.IsAuthenticated Then
				' force a login
				MyBase.HandleUnauthorizedRequest(filterContext)
			End If

			' they need to be enrolled in the course
			filterContext.Result = New RedirectToRouteResult(New RouteValueDictionary From { { "controller", "Enrolment" }, { "action", "Index" } })
		End Sub
	End Class
End Namespace